@@ -5,9 +5,8 @@ from esphome.components import binary_sensor
 from esphome.const import CONF_DATA, CONF_TRIGGER_ID, CONF_NBITS, CONF_ADDRESS, \
     CONF_COMMAND, CONF_CODE, CONF_PULSE_LENGTH, CONF_SYNC, CONF_ZERO, CONF_ONE, CONF_INVERTED, \
     CONF_PROTOCOL, CONF_GROUP, CONF_DEVICE, CONF_STATE, CONF_CHANNEL, CONF_FAMILY, CONF_REPEAT, \
-    CONF_WAIT_TIME, CONF_TIMES, CONF_TYPE_ID, CONF_CARRIER_FREQUENCY
+    CONF_WAIT_TIME, CONF_TIMES, CONF_TYPE_ID, CONF_CARRIER_FREQUENCY, CONF_RC_CODE_1, CONF_RC_CODE_2
 from esphome.core import coroutine
-from esphome.py_compat import string_types, text_type
 from esphome.util import Registry, SimpleRegistry
 
 AUTO_LOAD = ['binary_sensor']
@@ -52,7 +51,7 @@ def register_trigger(name, type, data_type):
         cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(type),
         cv.GenerateID(CONF_RECEIVER_ID): cv.use_id(RemoteReceiverBase),
     })
-    registerer = TRIGGER_REGISTRY.register('on_{}'.format(name), validator)
+    registerer = TRIGGER_REGISTRY.register(f'on_{name}', validator)
 
     def decorator(func):
         @coroutine
@@ -87,7 +86,7 @@ def validate_repeat(value):
     if isinstance(value, dict):
         return cv.Schema({
             cv.Required(CONF_TIMES): cv.templatable(cv.positive_int),
-            cv.Optional(CONF_WAIT_TIME, default='10ms'):
+            cv.Optional(CONF_WAIT_TIME, default='25ms'):
                 cv.templatable(cv.positive_time_period_microseconds),
         })(value)
     return validate_repeat({CONF_TIMES: value})
@@ -98,7 +97,7 @@ def register_action(name, type_, schema):
         cv.GenerateID(CONF_TRANSMITTER_ID): cv.use_id(RemoteTransmitterBase),
         cv.Optional(CONF_REPEAT): validate_repeat,
     })
-    registerer = automation.register_action('remote_transmitter.transmit_{}'.format(name),
+    registerer = automation.register_action(f'remote_transmitter.transmit_{name}',
                                             type_, validator)
 
     def decorator(func):
@@ -122,11 +121,11 @@ def register_action(name, type_, schema):
 
 
 def declare_protocol(name):
-    data = ns.struct('{}Data'.format(name))
-    binary_sensor_ = ns.class_('{}BinarySensor'.format(name), RemoteReceiverBinarySensorBase)
-    trigger = ns.class_('{}Trigger'.format(name), RemoteReceiverTrigger)
-    action = ns.class_('{}Action'.format(name), RemoteTransmitterActionBase)
-    dumper = ns.class_('{}Dumper'.format(name), RemoteTransmitterDumper)
+    data = ns.struct(f'{name}Data')
+    binary_sensor_ = ns.class_(f'{name}BinarySensor', RemoteReceiverBinarySensorBase)
+    trigger = ns.class_(f'{name}Trigger', RemoteReceiverTrigger)
+    action = ns.class_(f'{name}Action', RemoteTransmitterActionBase)
+    dumper = ns.class_(f'{name}Dumper', RemoteTransmitterDumper)
     return data, binary_sensor_, trigger, action, dumper
 
 
@@ -141,7 +140,7 @@ DUMPER_REGISTRY = Registry({
 
 
 def validate_dumpers(value):
-    if isinstance(value, string_types) and value.lower() == 'all':
+    if isinstance(value, str) and value.lower() == 'all':
         return validate_dumpers(list(DUMPER_REGISTRY.keys()))
     return cv.validate_registry('dumper', DUMPER_REGISTRY)(value)
 
@@ -289,6 +288,42 @@ def nec_action(var, config, args):
     cg.add(var.set_command(template_))
 
 
+# Pioneer
+(PioneerData, PioneerBinarySensor, PioneerTrigger, PioneerAction,
+ PioneerDumper) = declare_protocol('Pioneer')
+PIONEER_SCHEMA = cv.Schema({
+    cv.Required(CONF_RC_CODE_1): cv.hex_uint16_t,
+    cv.Optional(CONF_RC_CODE_2, default=0): cv.hex_uint16_t,
+})
+
+
+@register_binary_sensor('pioneer', PioneerBinarySensor, PIONEER_SCHEMA)
+def pioneer_binary_sensor(var, config):
+    cg.add(var.set_data(cg.StructInitializer(
+        PioneerData,
+        ('rc_code_1', config[CONF_RC_CODE_1]),
+        ('rc_code_2', config[CONF_RC_CODE_2]),
+    )))
+
+
+@register_trigger('pioneer', PioneerTrigger, PioneerData)
+def pioneer_trigger(var, config):
+    pass
+
+
+@register_dumper('pioneer', PioneerDumper)
+def pioneer_dumper(var, config):
+    pass
+
+
+@register_action('pioneer', PioneerAction, PIONEER_SCHEMA)
+def pioneer_action(var, config, args):
+    template_ = yield cg.templatable(config[CONF_RC_CODE_1], args, cg.uint16)
+    cg.add(var.set_rc_code_1(template_))
+    template_ = yield cg.templatable(config[CONF_RC_CODE_2], args, cg.uint16)
+    cg.add(var.set_rc_code_2(template_))
+
+
 # Sony
 SonyData, SonyBinarySensor, SonyTrigger, SonyAction, SonyDumper = declare_protocol('Sony')
 SONY_SCHEMA = cv.Schema({
@@ -385,7 +420,7 @@ def raw_action(var, config, args):
 RC5Data, RC5BinarySensor, RC5Trigger, RC5Action, RC5Dumper = declare_protocol('RC5')
 RC5_SCHEMA = cv.Schema({
     cv.Required(CONF_ADDRESS): cv.All(cv.hex_int, cv.Range(min=0, max=0x1F)),
-    cv.Required(CONF_COMMAND): cv.All(cv.hex_int, cv.Range(min=0, max=0x3F)),
+    cv.Required(CONF_COMMAND): cv.All(cv.hex_int, cv.Range(min=0, max=0x7F)),
 })
 
 
@@ -432,12 +467,12 @@ RC_SWITCH_PROTOCOL_SCHEMA = cv.Any(
 
 
 def validate_rc_switch_code(value):
-    if not isinstance(value, (str, text_type)):
+    if not isinstance(value, (str, str)):
         raise cv.Invalid("All RCSwitch codes must be in quotes ('')")
     for c in value:
         if c not in ('0', '1'):
-            raise cv.Invalid(u"Invalid RCSwitch code character '{}'. Only '0' and '1' are allowed"
-                             u"".format(c))
+            raise cv.Invalid("Invalid RCSwitch code character '{}'. Only '0' and '1' are allowed"
+                             "".format(c))
     if len(value) > 64:
         raise cv.Invalid("Maximum length for RCSwitch codes is 64, code '{}' has length {}"
                          "".format(value, len(value)))
@@ -447,7 +482,7 @@ def validate_rc_switch_code(value):
 
 
 def validate_rc_switch_raw_code(value):
-    if not isinstance(value, (str, text_type)):
+    if not isinstance(value, (str, str)):
         raise cv.Invalid("All RCSwitch raw codes must be in quotes ('')")
     for c in value:
         if c not in ('0', '1', 'x'):
@@ -505,13 +540,15 @@ RC_SWITCH_TYPE_D_SCHEMA = cv.Schema({
 RC_SWITCH_TRANSMITTER = cv.Schema({
     cv.Optional(CONF_REPEAT, default={CONF_TIMES: 5}): cv.Schema({
         cv.Required(CONF_TIMES): cv.templatable(cv.positive_int),
-        cv.Optional(CONF_WAIT_TIME, default='10ms'):
+        cv.Optional(CONF_WAIT_TIME, default='0us'):
             cv.templatable(cv.positive_time_period_microseconds),
     }),
 })
 
 rc_switch_protocols = ns.rc_switch_protocols
+RCSwitchData = ns.struct('RCSwitchData')
 RCSwitchBase = ns.class_('RCSwitchBase')
+RCSwitchTrigger = ns.class_('RCSwitchTrigger', RemoteReceiverTrigger)
 RCSwitchDumper = ns.class_('RCSwitchDumper', RemoteTransmitterDumper)
 RCSwitchRawAction = ns.class_('RCSwitchRawAction', RemoteTransmitterActionBase)
 RCSwitchTypeAAction = ns.class_('RCSwitchTypeAAction', RemoteTransmitterActionBase)
@@ -607,6 +644,11 @@ def rc_switch_type_d_action(var, config, args):
     cg.add(var.set_state((yield cg.templatable(config[CONF_STATE], args, bool))))
 
 
+@register_trigger('rc_switch', RCSwitchTrigger, RCSwitchData)
+def rc_switch_trigger(var, config):
+    pass
+
+
 @register_dumper('rc_switch', RCSwitchDumper)
 def rc_switch_dumper(var, config):
     pass
