@@ -13,16 +13,18 @@ class RGBWLightOutput : public light::LightOutput {
   void set_green(output::FloatOutput *green) { green_ = green; }
   void set_blue(output::FloatOutput *blue) { blue_ = blue; }
   void set_white(output::FloatOutput *white) { white_ = white; }
+  void set_color_interlock(bool color_interlock) { color_interlock_ = color_interlock; }
   light::LightTraits get_traits() override {
     auto traits = light::LightTraits();
     traits.set_supports_brightness(true);
+    traits.set_supports_color_interlock(this->color_interlock_);
     traits.set_supports_rgb(true);
     traits.set_supports_rgb_white_value(true);
     return traits;
   }
   void write_state(light::LightState *state) override {
     float red, green, blue, white;
-    state->current_values_as_rgbw(&red, &green, &blue, &white);
+    state->current_values_as_rgbw(&red, &green, &blue, &white, this->color_interlock_);
     this->red_->set_level(red);
     this->green_->set_level(green);
     this->blue_->set_level(blue);
@@ -34,6 +36,7 @@ class RGBWLightOutput : public light::LightOutput {
   output::FloatOutput *green_;
   output::FloatOutput *blue_;
   output::FloatOutput *white_;
+  bool color_interlock_{false};
 };
 
 }  // namespace rgbw
