@@ -13,6 +13,8 @@ class SCD30Component : public PollingComponent, public i2c::I2CDevice {
   void set_co2_sensor(sensor::Sensor *co2) { co2_sensor_ = co2; }
   void set_humidity_sensor(sensor::Sensor *humidity) { humidity_sensor_ = humidity; }
   void set_temperature_sensor(sensor::Sensor *temperature) { temperature_sensor_ = temperature; }
+  void set_automatic_self_calibration(bool asc) { enable_asc_ = asc; }
+  void set_altitude_compensation(uint16_t altitude) { altitude_compensation_ = altitude; }
 
   void setup() override;
   void update() override;
@@ -21,6 +23,7 @@ class SCD30Component : public PollingComponent, public i2c::I2CDevice {
 
  protected:
   bool write_command_(uint16_t command);
+  bool write_command_(uint16_t command, uint16_t data);
   bool read_data_(uint16_t *data, uint8_t len);
   uint8_t sht_crc_(uint8_t data1, uint8_t data2);
 
@@ -30,6 +33,8 @@ class SCD30Component : public PollingComponent, public i2c::I2CDevice {
     MEASUREMENT_INIT_FAILED,
     UNKNOWN
   } error_code_{UNKNOWN};
+  bool enable_asc_{true};
+  uint16_t altitude_compensation_{0xFFFF};
 
   sensor::Sensor *co2_sensor_{nullptr};
   sensor::Sensor *humidity_sensor_{nullptr};
