@@ -18,7 +18,9 @@ class TuyaClimate : public climate::Climate, public Component {
   void set_current_temperature_id(uint8_t current_temperature_id) {
     this->current_temperature_id_ = current_temperature_id;
   }
-  // void set_eco_mode_id(uint8_t eco_mode_id) { this->eco_mode_id_ = eco_mode_id; }
+  void set_temperature_multiplier(float temperature_multiplier) {
+    this->temperature_multiplier_ = temperature_multiplier;
+  }
 
   void set_tuya_parent(Tuya *parent) { this->parent_ = parent; }
 
@@ -38,7 +40,7 @@ class TuyaClimate : public climate::Climate, public Component {
   optional<uint8_t> switch_id_{};
   optional<uint8_t> target_temperature_id_{};
   optional<uint8_t> current_temperature_id_{};
-  // optional<uint8_t> eco_mode_id_{};
+  float temperature_multiplier_{1.0f};
 };
 
 }  // namespace tuya
