@@ -1,42 +1,47 @@
+import abc
 import inspect
-
 import math
 
 # pylint: disable=unused-import, wrong-import-order
-from typing import Any, Generator, List, Optional, Tuple, Type, Union, Dict, Callable  # noqa
+from typing import Any, Generator, List, Optional, Tuple, Type, Union, Sequence
 
 from esphome.core import (  # noqa
     CORE, HexInt, ID, Lambda, TimePeriod, TimePeriodMicroseconds,
     TimePeriodMilliseconds, TimePeriodMinutes, TimePeriodSeconds, coroutine, Library, Define,
     EnumValue)
 from esphome.helpers import cpp_string_escape, indent_all_but_first_and_last
-from esphome.py_compat import integer_types, string_types, text_type
 from esphome.util import OrderedDict
 
 
-class Expression(object):
+class Expression(abc.ABC):
+    __slots__ = ()
+
+    @abc.abstractmethod
     def __str__(self):
-        raise NotImplementedError
+        """
+        Convert expression into C++ code
+        """
 
 
-SafeExpType = Union[Expression, bool, str, text_type, int, float, TimePeriod,
-                    Type[bool], Type[int], Type[float], List[Any]]
+SafeExpType = Union[Expression, bool, str, str, int, float, TimePeriod,
+                    Type[bool], Type[int], Type[float], Sequence[Any]]
 
 
 class RawExpression(Expression):
-    def __init__(self, text):  # type: (Union[str, unicode]) -> None
-        super(RawExpression, self).__init__()
+    __slots__ = ("text", )
+
+    def __init__(self, text: str):
         self.text = text
 
     def __str__(self):
-        return str(self.text)
+        return self.text
 
 
-# pylint: disable=redefined-builtin
 class AssignmentExpression(Expression):
-    def __init__(self, type, modifier, name, rhs, obj):
-        super(AssignmentExpression, self).__init__()
-        self.type = type
+    __slots__ = ("type", "modifier", "name", "rhs", "obj")
+
+    def __init__(self, type_, modifier, name, rhs, obj):
+        self.type = type_
         self.modifier = modifier
         self.name = name
         self.rhs = safe_exp(rhs)
@@ -44,24 +49,26 @@ class AssignmentExpression(Expression):
 
     def __str__(self):
         if self.type is None:
-            return u"{} = {}".format(self.name, self.rhs)
-        return u"{} {}{} = {}".format(self.type, self.modifier, self.name, self.rhs)
+            return f"{self.name} = {self.rhs}"
+        return f"{self.type} {self.modifier}{self.name} = {self.rhs}"
 
 
 class VariableDeclarationExpression(Expression):
-    def __init__(self, type, modifier, name):
-        super(VariableDeclarationExpression, self).__init__()
-        self.type = type
+    __slots__ = ("type", "modifier", "name")
+
+    def __init__(self, type_, modifier, name):
+        self.type = type_
         self.modifier = modifier
         self.name = name
 
     def __str__(self):
-        return u"{} {}{}".format(self.type, self.modifier, self.name)
+        return f"{self.type} {self.modifier}{self.name}"
 
 
 class ExpressionList(Expression):
-    def __init__(self, *args):
-        super(ExpressionList, self).__init__()
+    __slots__ = ("args", )
+
+    def __init__(self, *args: Optional[SafeExpType]):
         # Remove every None on end
         args = list(args)
         while args and args[-1] is None:
@@ -69,7 +76,7 @@ class ExpressionList(Expression):
         self.args = [safe_exp(arg) for arg in args]
 
     def __str__(self):
-        text = u", ".join(text_type(x) for x in self.args)
+        text = ", ".join(str(x) for x in self.args)
         return indent_all_but_first_and_last(text)
 
     def __iter__(self):
@@ -77,20 +84,22 @@ class ExpressionList(Expression):
 
 
 class TemplateArguments(Expression):
-    def __init__(self, *args):  # type: (*SafeExpType) -> None
-        super(TemplateArguments, self).__init__()
+    __slots__ = ("args", )
+
+    def __init__(self, *args: SafeExpType):
         self.args = ExpressionList(*args)
 
     def __str__(self):
-        return u'<{}>'.format(self.args)
+        return f'<{self.args}>'
 
     def __iter__(self):
         return iter(self.args)
 
 
 class CallExpression(Expression):
-    def __init__(self, base, *args):  # type: (Expression, *SafeExpType) -> None
-        super(CallExpression, self).__init__()
+    __slots__ = ("base", "template_args", "args")
+
+    def __init__(self, base: Expression, *args: SafeExpType):
         self.base = base
         if args and isinstance(args[0], TemplateArguments):
             self.template_args = args[0]
@@ -101,14 +110,16 @@ class CallExpression(Expression):
 
     def __str__(self):
         if self.template_args is not None:
-            return u'{}{}({})'.format(self.base, self.template_args, self.args)
-        return u'{}({})'.format(self.base, self.args)
+            return f'{self.base}{self.template_args}({self.args})'
+        return f'{self.base}({self.args})'
 
 
 class StructInitializer(Expression):
-    def __init__(self, base, *args):  # type: (Expression, *Tuple[str, SafeExpType]) -> None
-        super(StructInitializer, self).__init__()
+    __slots__ = ("base", "args")
+
+    def __init__(self, base: Expression, *args: Tuple[str, Optional[SafeExpType]]):
         self.base = base
+        # TODO: args is always a Tuple, is this check required?
         if not isinstance(args, OrderedDict):
             args = OrderedDict(args)
         self.args = OrderedDict()
@@ -119,17 +130,18 @@ class StructInitializer(Expression):
             self.args[key] = exp
 
     def __str__(self):
-        cpp = u'{}{{\n'.format(self.base)
+        cpp = f'{self.base}{{\n'
         for key, value in self.args.items():
-            cpp += u'  .{} = {},\n'.format(key, value)
-        cpp += u'}'
+            cpp += f'  .{key} = {value},\n'
+        cpp += '}'
         return cpp
 
 
 class ArrayInitializer(Expression):
-    def __init__(self, *args, **kwargs):  # type: (*Any, **Any) -> None
-        super(ArrayInitializer, self).__init__()
-        self.multiline = kwargs.get('multiline', False)
+    __slots__ = ("multiline", "args")
+
+    def __init__(self, *args: Any, multiline: bool = False):
+        self.multiline = multiline
         self.args = []
         for arg in args:
             if arg is None:
@@ -139,30 +151,32 @@ class ArrayInitializer(Expression):
 
     def __str__(self):
         if not self.args:
-            return u'{}'
+            return '{}'
         if self.multiline:
-            cpp = u'{\n'
+            cpp = '{\n'
             for arg in self.args:
-                cpp += u'  {},\n'.format(arg)
-            cpp += u'}'
+                cpp += f'  {arg},\n'
+            cpp += '}'
         else:
-            cpp = u'{' + u', '.join(str(arg) for arg in self.args) + u'}'
+            cpp = '{' + ', '.join(str(arg) for arg in self.args) + '}'
         return cpp
 
 
 class ParameterExpression(Expression):
-    def __init__(self, type, id):
-        super(ParameterExpression, self).__init__()
-        self.type = safe_exp(type)
-        self.id = id
+    __slots__ = ("type", "id")
+
+    def __init__(self, type_, id_):
+        self.type = safe_exp(type_)
+        self.id = id_
 
     def __str__(self):
-        return u"{} {}".format(self.type, self.id)
+        return f"{self.type} {self.id}"
 
 
 class ParameterListExpression(Expression):
-    def __init__(self, *parameters):
-        super(ParameterListExpression, self).__init__()
+    __slots__ = ("parameters", )
+
+    def __init__(self, *parameters: Union[ParameterExpression, Tuple[SafeExpType, str]]):
         self.parameters = []
         for parameter in parameters:
             if not isinstance(parameter, ParameterExpression):
@@ -170,12 +184,13 @@ class ParameterListExpression(Expression):
             self.parameters.append(parameter)
 
     def __str__(self):
-        return u", ".join(text_type(x) for x in self.parameters)
+        return ", ".join(str(x) for x in self.parameters)
 
 
 class LambdaExpression(Expression):
-    def __init__(self, parts, parameters, capture='=', return_type=None):
-        super(LambdaExpression, self).__init__()
+    __slots__ = ("parts", "parameters", "capture", "return_type")
+
+    def __init__(self, parts, parameters, capture: str = '=', return_type=None):
         self.parts = parts
         if not isinstance(parameters, ParameterListExpression):
             parameters = ParameterListExpression(*parameters)
@@ -184,58 +199,66 @@ class LambdaExpression(Expression):
         self.return_type = safe_exp(return_type) if return_type is not None else None
 
     def __str__(self):
-        cpp = u'[{}]({})'.format(self.capture, self.parameters)
+        cpp = f'[{self.capture}]({self.parameters})'
         if self.return_type is not None:
-            cpp += u' -> {}'.format(self.return_type)
-        cpp += u' {{\n{}\n}}'.format(self.content)
+            cpp += f' -> {self.return_type}'
+        cpp += f' {{\n{self.content}\n}}'
         return indent_all_but_first_and_last(cpp)
 
     @property
     def content(self):
-        return u''.join(text_type(part) for part in self.parts)
+        return ''.join(str(part) for part in self.parts)
 
 
-class Literal(Expression):
-    def __str__(self):
-        raise NotImplementedError
+# pylint: disable=abstract-method
+class Literal(Expression, metaclass=abc.ABCMeta):
+    __slots__ = ()
 
 
 class StringLiteral(Literal):
-    def __init__(self, string):  # type: (Union[str, unicode]) -> None
-        super(StringLiteral, self).__init__()
+    __slots__ = ("string", )
+
+    def __init__(self, string: str):
+        super().__init__()
         self.string = string
 
     def __str__(self):
-        return u'{}'.format(cpp_string_escape(self.string))
+        return cpp_string_escape(self.string)
 
 
 class IntLiteral(Literal):
-    def __init__(self, i):  # type: (Union[int, long]) -> None
-        super(IntLiteral, self).__init__()
+    __slots__ = ("i", )
+
+    def __init__(self, i: int):
+        super().__init__()
         self.i = i
 
     def __str__(self):
         if self.i > 4294967295:
-            return u'{}ULL'.format(self.i)
+            return f'{self.i}ULL'
         if self.i > 2147483647:
-            return u'{}UL'.format(self.i)
+            return f'{self.i}UL'
         if self.i < -2147483648:
-            return u'{}LL'.format(self.i)
-        return text_type(self.i)
+            return f'{self.i}LL'
+        return str(self.i)
 
 
 class BoolLiteral(Literal):
-    def __init__(self, binary):  # type: (bool) -> None
-        super(BoolLiteral, self).__init__()
+    __slots__ = ("binary", )
+
+    def __init__(self, binary: bool):
+        super().__init__()
         self.binary = binary
 
     def __str__(self):
-        return u"true" if self.binary else u"false"
+        return "true" if self.binary else "false"
 
 
 class HexIntLiteral(Literal):
-    def __init__(self, i):  # type: (int) -> None
-        super(HexIntLiteral, self).__init__()
+    __slots__ = ("i", )
+
+    def __init__(self, i: int):
+        super().__init__()
         self.i = HexInt(i)
 
     def __str__(self):
@@ -243,21 +266,19 @@ class HexIntLiteral(Literal):
 
 
 class FloatLiteral(Literal):
-    def __init__(self, value):  # type: (float) -> None
-        super(FloatLiteral, self).__init__()
-        self.float_ = value
+    __slots__ = ("f", )
+
+    def __init__(self, value: float):
+        super().__init__()
+        self.f = value
 
     def __str__(self):
-        if math.isnan(self.float_):
-            return u"NAN"
-        return u"{}f".format(self.float_)
+        if math.isnan(self.f):
+            return "NAN"
+        return f"{self.f}f"
 
 
-# pylint: disable=bad-continuation
-def safe_exp(
-        obj  # type: Union[Expression, bool, str, unicode, int, long, float, TimePeriod, list]
-             ):
-    # type: (...) -> Expression
+def safe_exp(obj: SafeExpType) -> Expression:
     """Try to convert obj to an expression by automatically converting native python types to
     expressions/literals.
     """
@@ -269,11 +290,11 @@ def safe_exp(
         return safe_exp(obj.enum_value)
     if isinstance(obj, bool):
         return BoolLiteral(obj)
-    if isinstance(obj, string_types):
+    if isinstance(obj, str):
         return StringLiteral(obj)
     if isinstance(obj, HexInt):
         return HexIntLiteral(obj)
-    if isinstance(obj, integer_types):
+    if isinstance(obj, int):
         return IntLiteral(obj)
     if isinstance(obj, float):
         return FloatLiteral(obj)
@@ -294,25 +315,28 @@ def safe_exp(
     if obj is float:
         return float_
     if isinstance(obj, ID):
-        raise ValueError(u"Object {} is an ID. Did you forget to register the variable?"
-                         u"".format(obj))
+        raise ValueError("Object {} is an ID. Did you forget to register the variable?"
+                         "".format(obj))
     if inspect.isgenerator(obj):
-        raise ValueError(u"Object {} is a coroutine. Did you forget to await the expression with "
-                         u"'yield'?".format(obj))
-    raise ValueError(u"Object is not an expression", obj)
+        raise ValueError("Object {} is a coroutine. Did you forget to await the expression with "
+                         "'yield'?".format(obj))
+    raise ValueError("Object is not an expression", obj)
 
 
-class Statement(object):
-    def __init__(self):
-        pass
+class Statement(abc.ABC):
+    __slots__ = ()
 
+    @abc.abstractmethod
     def __str__(self):
-        raise NotImplementedError
+        """
+        Convert statement into C++ code
+        """
 
 
 class RawStatement(Statement):
-    def __init__(self, text):
-        super(RawStatement, self).__init__()
+    __slots__ = ("text", )
+
+    def __init__(self, text: str):
         self.text = text
 
     def __str__(self):
@@ -320,124 +344,115 @@ class RawStatement(Statement):
 
 
 class ExpressionStatement(Statement):
+    __slots__ = ("expression", )
+
     def __init__(self, expression):
-        super(ExpressionStatement, self).__init__()
         self.expression = safe_exp(expression)
 
     def __str__(self):
-        return u"{};".format(self.expression)
+        return f"{self.expression};"
 
 
 class LineComment(Statement):
-    def __init__(self, value):  # type: (unicode) -> None
-        super(LineComment, self).__init__()
-        self._value = value
+    __slots__ = ("value", )
+
+    def __init__(self, value: str):
+        self.value = value
 
     def __str__(self):
-        parts = self._value.split(u'\n')
-        parts = [u'// {}'.format(x) for x in parts]
-        return u'\n'.join(parts)
+        parts = self.value.split('\n')
+        parts = [f'// {x}' for x in parts]
+        return '\n'.join(parts)
 
 
 class ProgmemAssignmentExpression(AssignmentExpression):
-    def __init__(self, type, name, rhs, obj):
-        super(ProgmemAssignmentExpression, self).__init__(
-            type, '', name, rhs, obj
-        )
+    __slots__ = ()
+
+    def __init__(self, type_, name, rhs, obj):
+        super().__init__(type_, '', name, rhs, obj)
 
     def __str__(self):
-        type_ = self.type
-        return u"static const {} {}[] PROGMEM = {}".format(type_, self.name, self.rhs)
+        return f"static const {self.type} {self.name}[] PROGMEM = {self.rhs}"
 
 
-def progmem_array(id, rhs):
+def progmem_array(id_, rhs) -> "MockObj":
     rhs = safe_exp(rhs)
-    obj = MockObj(id, u'.')
-    assignment = ProgmemAssignmentExpression(id.type, id, rhs, obj)
+    obj = MockObj(id_, '.')
+    assignment = ProgmemAssignmentExpression(id_.type, id_, rhs, obj)
     CORE.add(assignment)
-    CORE.register_variable(id, obj)
+    CORE.register_variable(id_, obj)
     return obj
 
 
-def statement(expression):  # type: (Union[Expression, Statement]) -> Statement
+def statement(expression: Union[Expression, Statement]) -> Statement:
+    """Convert expression into a statement unless is already a statement.
+    """
     if isinstance(expression, Statement):
         return expression
     return ExpressionStatement(expression)
 
 
-def variable(id,  # type: ID
-             rhs,  # type: SafeExpType
-             type=None  # type: MockObj
-             ):
-    # type: (...) -> MockObj
+def variable(id_: ID, rhs: SafeExpType, type_: "MockObj" = None) -> "MockObj":
     """Declare a new variable (not pointer type) in the code generation.
 
-    :param id: The ID used to declare the variable.
+    :param id_: The ID used to declare the variable.
     :param rhs: The expression to place on the right hand side of the assignment.
-    :param type: Manually define a type for the variable, only use this when it's not possible
+    :param type_: Manually define a type for the variable, only use this when it's not possible
       to do so during config validation phase (for example because of template arguments).
 
     :returns The new variable as a MockObj.
     """
-    assert isinstance(id, ID)
+    assert isinstance(id_, ID)
     rhs = safe_exp(rhs)
-    obj = MockObj(id, u'.')
-    if type is not None:
-        id.type = type
-    assignment = AssignmentExpression(id.type, '', id, rhs, obj)
+    obj = MockObj(id_, '.')
+    if type_ is not None:
+        id_.type = type_
+    assignment = AssignmentExpression(id_.type, '', id_, rhs, obj)
     CORE.add(assignment)
-    CORE.register_variable(id, obj)
+    CORE.register_variable(id_, obj)
     return obj
 
 
-def Pvariable(id,  # type: ID
-              rhs,  # type: SafeExpType
-              type=None  # type: MockObj
-              ):
-    # type: (...) -> MockObj
+def Pvariable(id_: ID, rhs: SafeExpType, type_: "MockObj" = None) -> "MockObj":
     """Declare a new pointer variable in the code generation.
 
-    :param id: The ID used to declare the variable.
+    :param id_: The ID used to declare the variable.
     :param rhs: The expression to place on the right hand side of the assignment.
-    :param type: Manually define a type for the variable, only use this when it's not possible
+    :param type_: Manually define a type for the variable, only use this when it's not possible
       to do so during config validation phase (for example because of template arguments).
 
     :returns The new variable as a MockObj.
     """
     rhs = safe_exp(rhs)
-    obj = MockObj(id, u'->')
-    if type is not None:
-        id.type = type
-    decl = VariableDeclarationExpression(id.type, '*', id)
+    obj = MockObj(id_, '->')
+    if type_ is not None:
+        id_.type = type_
+    decl = VariableDeclarationExpression(id_.type, '*', id_)
     CORE.add_global(decl)
-    assignment = AssignmentExpression(None, None, id, rhs, obj)
+    assignment = AssignmentExpression(None, None, id_, rhs, obj)
     CORE.add(assignment)
-    CORE.register_variable(id, obj)
+    CORE.register_variable(id_, obj)
     return obj
 
 
-def new_Pvariable(id,  # type: ID
-                  *args  # type: *SafeExpType
-                  ):
+def new_Pvariable(id_: ID, *args: SafeExpType) -> Pvariable:
     """Declare a new pointer variable in the code generation by calling it's constructor
     with the given arguments.
 
-    :param id: The ID used to declare the variable (also specifies the type).
+    :param id_: The ID used to declare the variable (also specifies the type).
     :param args: The values to pass to the constructor.
 
     :returns The new variable as a MockObj.
     """
     if args and isinstance(args[0], TemplateArguments):
-        id = id.copy()
-        id.type = id.type.template(args[0])
+        id_ = id_.copy()
+        id_.type = id_.type.template(args[0])
         args = args[1:]
-    rhs = id.type.new(*args)
-    return Pvariable(id, rhs)
+    rhs = id_.type.new(*args)
+    return Pvariable(id_, rhs)
 
 
-def add(expression,  # type: Union[Expression, Statement]
-        ):
-    # type: (...) -> None
+def add(expression: Union[Expression, Statement]):
     """Add an expression to the codegen section.
 
     After this is called, the given given expression will
@@ -446,17 +461,12 @@ def add(expression,  # type: Union[Expression, Statement]
     CORE.add(expression)
 
 
-def add_global(expression,  # type: Union[SafeExpType, Statement]
-               ):
-    # type: (...) -> None
+def add_global(expression: Union[SafeExpType, Statement]):
     """Add an expression to the codegen global storage (above setup())."""
     CORE.add_global(expression)
 
 
-def add_library(name,  # type: str
-                version  # type: Optional[str]
-                ):
-    # type: (...) -> None
+def add_library(name: str, version: Optional[str]):
     """Add a library to the codegen library storage.
 
     :param name: The name of the library (for example 'AsyncTCP')
@@ -465,17 +475,12 @@ def add_library(name,  # type: str
     CORE.add_library(Library(name, version))
 
 
-def add_build_flag(build_flag,  # type: str
-                   ):
-    # type: (...) -> None
+def add_build_flag(build_flag: str):
     """Add a global build flag to the compiler flags."""
     CORE.add_build_flag(build_flag)
 
 
-def add_define(name,  # type: str
-               value=None,  # type: Optional[SafeExpType]
-               ):
-    # type: (...) -> None
+def add_define(name: str, value: SafeExpType = None):
     """Add a global define to the auto-generated defines.h file.
 
     Optionally define a value to set this define to.
@@ -487,42 +492,40 @@ def add_define(name,  # type: str
 
 
 @coroutine
-def get_variable(id):  # type: (ID) -> Generator[MockObj]
+def get_variable(id_: ID) -> Generator["MockObj", None, None]:
     """
     Wait for the given ID to be defined in the code generation and
     return it as a MockObj.
 
     This is a coroutine, you need to await it with a 'yield' expression!
 
-    :param id: The ID to retrieve
+    :param id_: The ID to retrieve
     :return: The variable as a MockObj.
     """
-    var = yield CORE.get_variable(id)
+    var = yield CORE.get_variable(id_)
     yield var
 
 
 @coroutine
-def get_variable_with_full_id(id):  # type: (ID) -> Generator[ID, MockObj]
+def get_variable_with_full_id(id_: ID) -> Generator[Tuple[ID, "MockObj"], None, None]:
     """
     Wait for the given ID to be defined in the code generation and
     return it as a MockObj.
 
     This is a coroutine, you need to await it with a 'yield' expression!
 
-    :param id: The ID to retrieve
+    :param id_: The ID to retrieve
     :return: The variable as a MockObj.
     """
-    full_id, var = yield CORE.get_variable_with_full_id(id)
+    full_id, var = yield CORE.get_variable_with_full_id(id_)
     yield full_id, var
 
 
 @coroutine
-def process_lambda(value,  # type: Lambda
-                   parameters,  # type: List[Tuple[SafeExpType, str]]
-                   capture='=',  # type: str
-                   return_type=None  # type: Optional[SafeExpType]
-                   ):
-    # type: (...) -> Generator[LambdaExpression]
+def process_lambda(
+        value: Lambda, parameters: List[Tuple[SafeExpType, str]],
+        capture: str = '=', return_type: SafeExpType = None
+) -> Generator[LambdaExpression, None, None]:
     """Process the given lambda value into a LambdaExpression.
 
     This is a coroutine because lambdas can depend on other IDs,
@@ -561,11 +564,10 @@ def is_template(value):
 
 
 @coroutine
-def templatable(value,  # type: Any
-                args,  # type: List[Tuple[SafeExpType, str]]
-                output_type,  # type: Optional[SafeExpType],
-                to_exp=None  # type: Optional[Any]
-                ):
+def templatable(value: Any,
+                args: List[Tuple[SafeExpType, str]],
+                output_type: Optional[SafeExpType],
+                to_exp: Any = None):
     """Generate code for a templatable config option.
 
     If `value` is a templated value, the lambda expression is returned.
@@ -594,78 +596,79 @@ class MockObj(Expression):
 
     Mostly consists of magic methods that allow ESPHome's codegen syntax.
     """
-    def __init__(self, base, op=u'.'):
+    __slots__ = ("base", "op")
+
+    def __init__(self, base, op='.'):
         self.base = base
         self.op = op
-        super(MockObj, self).__init__()
 
-    def __getattr__(self, attr):  # type: (str) -> MockObj
-        next_op = u'.'
-        if attr.startswith(u'P') and self.op not in ['::', '']:
+    def __getattr__(self, attr: str) -> "MockObj":
+        next_op = '.'
+        if attr.startswith('P') and self.op not in ['::', '']:
             attr = attr[1:]
-            next_op = u'->'
-        if attr.startswith(u'_'):
+            next_op = '->'
+        if attr.startswith('_'):
             attr = attr[1:]
-        return MockObj(u'{}{}{}'.format(self.base, self.op, attr), next_op)
+        return MockObj(f'{self.base}{self.op}{attr}', next_op)
 
     def __call__(self, *args):  # type: (SafeExpType) -> MockObj
         call = CallExpression(self.base, *args)
         return MockObj(call, self.op)
 
-    def __str__(self):  # type: () -> unicode
-        return text_type(self.base)
+    def __str__(self):
+        return str(self.base)
 
     def __repr__(self):
-        return u'MockObj<{}>'.format(text_type(self.base))
+        return 'MockObj<{}>'.format(str(self.base))
 
     @property
-    def _(self):  # type: () -> MockObj
-        return MockObj(u'{}{}'.format(self.base, self.op))
+    def _(self) -> "MockObj":
+        return MockObj(f'{self.base}{self.op}')
 
     @property
-    def new(self):  # type: () -> MockObj
-        return MockObj(u'new {}'.format(self.base), u'->')
+    def new(self) -> "MockObj":
+        return MockObj(f'new {self.base}', '->')
 
-    def template(self, *args):  # type: (*SafeExpType) -> MockObj
+    def template(self, *args: SafeExpType) -> "MockObj":
         if len(args) != 1 or not isinstance(args[0], TemplateArguments):
             args = TemplateArguments(*args)
         else:
             args = args[0]
-        return MockObj(u'{}{}'.format(self.base, args))
+        return MockObj(f'{self.base}{args}')
 
-    def namespace(self, name):  # type: (str) -> MockObj
-        return MockObj(u'{}{}'.format(self._, name), u'::')
+    def namespace(self, name: str) -> "MockObj":
+        return MockObj(f'{self._}{name}', '::')
 
-    def class_(self, name, *parents):  # type: (str, *MockObjClass) -> MockObjClass
+    def class_(self, name: str, *parents: "MockObjClass") -> "MockObjClass":
         op = '' if self.op == '' else '::'
-        return MockObjClass(u'{}{}{}'.format(self.base, op, name), u'.', parents=parents)
+        return MockObjClass(f'{self.base}{op}{name}', '.', parents=parents)
 
-    def struct(self, name):  # type: (str) -> MockObjClass
+    def struct(self, name: str) -> "MockObjClass":
         return self.class_(name)
 
-    def enum(self, name, is_class=False):  # type: (str, bool) -> MockObj
+    def enum(self, name: str, is_class: bool = False) -> "MockObj":
         return MockObjEnum(enum=name, is_class=is_class, base=self.base, op=self.op)
 
-    def operator(self, name):  # type: (str) -> MockObj
+    def operator(self, name: str) -> "MockObj":
         if name == 'ref':
-            return MockObj(u'{} &'.format(self.base), u'')
+            return MockObj(f'{self.base} &', '')
         if name == 'ptr':
-            return MockObj(u'{} *'.format(self.base), u'')
+            return MockObj(f'{self.base} *', '')
         if name == "const":
-            return MockObj(u'const {}'.format(self.base), u'')
-        raise NotImplementedError
+            return MockObj(f'const {self.base}', '')
+        raise ValueError("Expected one of ref, ptr, const.")
 
     @property
-    def using(self):  # type: () -> MockObj
+    def using(self) -> "MockObj":
         assert self.op == '::'
-        return MockObj(u'using namespace {}'.format(self.base))
+        return MockObj(f'using namespace {self.base}')
 
-    def __getitem__(self, item):  # type: (Union[str, Expression]) -> MockObj
-        next_op = u'.'
-        if isinstance(item, str) and item.startswith(u'P'):
+    def __getitem__(self, item: Union[str, Expression]) -> "MockObj":
+        next_op = '.'
+        if isinstance(item, str) and item.startswith('P'):
             item = item[1:]
-            next_op = u'->'
-        return MockObj(u'{}[{}]'.format(self.base, item), next_op)
+            next_op = '->'
+        return MockObj(f'{self.base}[{item}]', next_op)
 
 
 class MockObjEnum(MockObj):
@@ -679,13 +682,13 @@ class MockObjEnum(MockObj):
         kwargs['base'] = base
         MockObj.__init__(self, *args, **kwargs)
 
-    def __str__(self):  # type: () -> unicode
+    def __str__(self):
         if self._is_class:
-            return super(MockObjEnum, self).__str__()
-        return u'{}{}{}'.format(self.base, self.op, self._enum)
+            return super().__str__()
+        return f'{self.base}{self.op}{self._enum}'
 
     def __repr__(self):
-        return u'MockObj<{}>'.format(text_type(self.base))
+        return f'MockObj<{str(self.base)}>'
 
 
 class MockObjClass(MockObj):
@@ -700,7 +703,7 @@ class MockObjClass(MockObj):
             # pylint: disable=protected-access
             self._parents += paren._parents
 
-    def inherits_from(self, other):  # type: (MockObjClass) -> bool
+    def inherits_from(self, other: "MockObjClass") -> bool:
         if self == other:
             return True
         for parent in self._parents:
@@ -708,15 +711,14 @@ class MockObjClass(MockObj):
                 return True
         return False
 
-    def template(self, *args):
-        # type: (*SafeExpType) -> MockObjClass
+    def template(self, *args: SafeExpType) -> "MockObjClass":
         if len(args) != 1 or not isinstance(args[0], TemplateArguments):
             args = TemplateArguments(*args)
         else:
             args = args[0]
         new_parents = self._parents[:]
         new_parents.append(self)
-        return MockObjClass(u'{}{}'.format(self.base, args), parents=new_parents)
+        return MockObjClass(f'{self.base}{args}', parents=new_parents)
 
     def __repr__(self):
-        return u'MockObjClass<{}, parents={}>'.format(text_type(self.base), self._parents)
+        return f'MockObjClass<{str(self.base)}, parents={self._parents}>'
